"""
Contains the actions that GraphGrammar metamodel buttonclicks should perform

Author: Dieter De Hen
"""

import tkMessageBox
import os

from ATOM3TypeInfo       import *

def compileGG(self):
  """ 
  Compiles a GraphGrammar model
  """
  modelPathAndFile = self.statusbar.getState(self.statusbar.MODEL)[1][0]      
  modelPath, modelName = os.path.split(modelPathAndFile)
  modelName = modelName.split('.')[0]
  compiledModelPathAndFile = modelPathAndFile.split('_MDL.')[0] + "_GG.py"

  className = modelName.split('_MDL')[0] + '_GG'

  content = '"""\n'
  content += className + '\n'
  content += '\n'
  content += 'Author: Dieter De Hen\n'
  content += '"""\n'
  content += 'import random\n'
  content += '\n'
  content += 'from ContainerFunctionality import *\n'
  content += '\n'
  content += 'class ' + className + '(object):\n'
  content += '  \n'
  content += '  def __init__(self):\n'
  content += '    self.__model = ModelContainer()\n'
  content += '    \n'
  content += '    # Nodes\n'
  
  for node in self.ASGroot.listNodes:
    nodeType = node
    i = 0
    for nodeInstance in self.ASGroot.listNodes[node]:
      nodeInstanceName = 'obj' + str(nodeInstance.objectNumber)
      content += '    \n'
      content += '    self.__' + nodeInstanceName + ' = ModelNode({\'type\':\'' + nodeType + '\''
      for att in nodeInstance.generatedAttributes: 
        attValue = eval('self.ASGroot.listNodes[\'' + node + '\'][' + str(i) + '].' + att)
        if isinstance(attValue, ATOM3String):
          content += ', \'' + att + '\':\'' + attValue.strValue + '\''
        elif isinstance(attValue, ATOM3Integer):
          content += ', \'' + att + '\':' + str(attValue.intValue)
      content += '})\n'
      content += '    self.__model.get_nodes().append(self.__' + nodeInstanceName + ')\n'
      i = i + 1

  content += '    \n'
  content += '    # Node connections\n'
  content += '    \n'

  for node in self.ASGroot.listNodes:
    for nodeInstance in self.ASGroot.listNodes[node]:
      nodeInstanceName = 'obj' + str(nodeInstance.objectNumber)
      for nodeInstanceConnection in nodeInstance.in_connections_:
        nodeInstanceConnectionName = 'obj' + str(nodeInstanceConnection.objectNumber)
        content += '    self.__' + nodeInstanceName + '.get_inConnections().append(self.__' + nodeInstanceConnectionName + ')\n'
      for nodeInstanceConnection in nodeInstance.out_connections_:
        nodeInstanceConnectionName = 'obj' + str(nodeInstanceConnection.objectNumber)
        content += '    self.__' + nodeInstanceName + '.get_outConnections().append(self.__' + nodeInstanceConnectionName + ')\n'
      content += '    \n'
  content += '  def get_model(self):\n'
  content += '    return self.__model\n'
  content += '  \n'

  content += '  def get_GG( self ):\n'
  content += '    # Start nodes without in connection\n'
  content += '    startNodes = []\n'
  content += '    for s in self.__model.get_nodes():\n'
  content += '      if s.get_parameters()[\'type\'] == \'TP_Rule\':\n'
  content += '        inConnectionCount = 0\n'
  content += '        for inConnection in s.get_inConnections():\n'
  content += '          inConnectionCount = inConnectionCount + 1\n'
  content += '        if inConnectionCount == 0:\n'
  content += '          startNodes.append(s)\n'
  content += '    # If no start nodes found, use all nodes in model.\n'
  content += '    if len(startNodes) == 0:\n'
  content += '      for s in self.__model.get_nodes():\n'
  content += '        if s.get_parameters()[\'type\'] == \'TP_Rule\':\n'
  content += '          startNodes.append(s)\n'
  content += '    gg = random.sample(startNodes, 1)\n'
  content += '    return self.__buildGG(gg)\n'
  content += '  \n'
  content += '  def __buildGG( self , gg ):\n'
  content += '    lastIndex = len(gg)-1\n'
  content += '    if lastIndex >= 0:\n'
  content += '      if gg[lastIndex].get_parameters()[\'type\'] == \'TP_Rule\' and len(gg[lastIndex].get_outConnections()) > 0:\n'
  content += '        next = random.sample(gg[lastIndex].get_outConnections(), 1)\n'
  content += '        if next[0].get_parameters()[\'type\'] == \'TP_NextRule\' and len(next[0].get_outConnections()) > 0:\n'
  content += '          # a nextrule can only be connectect to one rule\n'
  content += '          if next[0].get_outConnections()[0].get_parameters()[\'type\'] == \'TP_Rule\':\n'
  content += '            gg.append(next[0].get_outConnections()[0]) \n'
  content += '            return self.__buildGG(gg)\n'
  content += '    return gg\n'
 
  file = open(compiledModelPathAndFile, 'w')
  
  file.write(content)
  
  file.close()
  
  tkMessageBox.showinfo("Compiling", 'Grammar compiled successfully to ' + className + '.py', parent = self)

  return True


