"""
Contains the actions that TP_Trans metamodel buttonclicks should perform

Author: Dieter De Hen
"""

import tkMessageBox
import os

from ATOM3TypeInfo       import *

class Magic:
  """
  Magic variables:
    If the TP_Trans formalism is re-named or modified, the following variables must 
    be changed accordingly. 
  """
  ASGname = 'ASG_TP_Trans'
  separatorName = 'TP_Separator'
  lhsPrefix = 'TP_LHS_'
  rhsPrefix = 'TP_RHS_'
  genericPrefix = 'GenericGraph'

class LStrings:
  """
  Localization Strings
  """
  def missingSeparator(self):
    return "- Add a separator\n"
  
  def maxOneSeparatorAllowed(self, found):
    return "- Only 1 separator is allowed. I found " + found + ".\n"
  
  def incorrectSeparatorCount(self, found):
    return "- Incorrect number of separators. I found " + found + ".\n"
  
  def collidingLabel(self, nodeName, label):
    return "- Colliding label '" + label + "' for instance of node '" + nodeName + "'.\n"
  
  def wrongRightSideSeparator(self, nodeName):
    return "- An instance of node '" + nodeName + "' is on the right-hand side of the separator.\n"
  
  def wrongLeftSideSeparator(self, nodeName):
    return "- An instance of node '" + nodeName + "' is on the left-hand side of the separator.\n"

def checkRule(self, msgBox = True):
  """ 
  Checks a TP rule model
  """
  
  errors = ""
  
  # Check if there exists a separator on the screen
  if not self.ASGroot.listNodes[Magic().separatorName]:
    errors += LStrings().missingSeparator()
  # Max one separator
  elif len(self.ASGroot.listNodes[Magic().separatorName]) > 1:
    errors += LStrings().maxOneSeparatorAllowed(str(len(self.ASGroot.listNodes[Magic().separatorName])))
  # Can this occur?
  elif len(self.ASGroot.listNodes[Magic().separatorName]) != 1:
    errors += LStrings().incorrectSeparatorCount(str(len(self.ASGroot.listNodes[Magic().separatorName])))
  # One separator exists
  else:
    separatorX = self.ASGroot.listNodes[Magic().separatorName][0].graphObject_.x
    
    lhsLabels = []
    rhsLabels = []
    
    for node in self.ASGroot.listNodes:
      if node.startswith(Magic().lhsPrefix):
        for nodeInstance in self.ASGroot.listNodes[node]:
          # Check the node has no colliding labels
          if nodeInstance.label.strValue in lhsLabels:
            errors += LStrings().collidingLabel(node, nodeInstance.label.strValue)
          # Check the node instance is on the correct side of the separator
          if nodeInstance.graphObject_.x >= separatorX:
            errors += LStrings().wrongRightSideSeparator(node)
          lhsLabels.append(nodeInstance.label.strValue)
      elif node.startswith(Magic().rhsPrefix):
        for nodeInstance in self.ASGroot.listNodes[node]:
          # Check the node has no colliding labels
          if nodeInstance.label.strValue in rhsLabels:
            errors += LStrings().collidingLabel(node, nodeInstance.label.strValue)
          # Check the node instance is on the correct side of the separator
          if nodeInstance.graphObject_.x <= separatorX:
            errors += LStrings().wrongLeftSideSeparator(node)
          rhsLabels.append(nodeInstance.label.strValue)
    
  if not errors:
    if msgBox:
      tkMessageBox.showinfo("Valid model", "This is a valid model.\n" + errors, parent = self)
    return True
  else:
    if msgBox:
      tkMessageBox.showerror("Not a valid model", "This is not a valid model:\n" + errors, parent = self)
    return False

def compileRule(self):
  """ 
  Compiles a TP rule model
  """
  modelPathAndFile = self.statusbar.getState(self.statusbar.MODEL)[1][0]      
  modelPath, modelName = os.path.split(modelPathAndFile)
  modelName = modelName.split('.')[0]
  compiledModelPathAndFile = modelPathAndFile.split('_MDL.')[0] + "_RULE.py"

  className = modelName.split('_MDL')[0] + '_RULE'

  content = '"""\n'
  content += className + '\n'
  content += '\n'
  content += 'Author: Dieter De Hen\n'
  content += '"""\n'
  content += '\n'
  content += 'import copy'
  content += '\n'
  content += 'from ContainerFunctionality import *\n'
  content += '\n'
  content += 'class ' + className + '(object):\n'
  content += '  \n'
  content += '  def __init__(self):\n'
  content += '    self.__lhsModel = ModelContainer()\n'
  content += '    self.__rhsModel = ModelContainer()\n'
  content += '    \n'
  content += '    # LHS nodes\n'
  
  for node in self.ASGroot.listNodes:
    if node.startswith(Magic().lhsPrefix):
      nodeType = node.split(Magic().lhsPrefix)[1]
      i = 0
      for nodeInstance in self.ASGroot.listNodes[node]:
        nodeInstanceName = 'obj' + str(nodeInstance.objectNumber)
        content += '    \n'
        content += '    self.__' + nodeInstanceName + ' = ModelNode({\'type\':\'' + nodeType + '\''
        for att in nodeInstance.generatedAttributes: 
          attValue = eval('self.ASGroot.listNodes[\'' + node + '\'][' + str(i) + '].' + att)
          if isinstance(attValue, ATOM3String):
             content += ', \'' + att + '\':\'' + attValue.strValue + '\''
          elif isinstance(attValue, ATOM3Integer):
             content += ', \'' + att + '\':' + str(attValue.intValue)
        content += '})\n'
        content += '    self.__lhsModel.get_nodes().append(self.__' + nodeInstanceName + ')\n'
        i = i + 1

  content += '    \n'
  content += '    # RHS nodes\n'
  
  for node in self.ASGroot.listNodes:
    if node.startswith(Magic().rhsPrefix):
      nodeType = node.split(Magic().rhsPrefix)[1]
      i = 0
      for nodeInstance in self.ASGroot.listNodes[node]:
        nodeInstanceName = 'obj' + str(nodeInstance.objectNumber)
        content += '    \n'
        content += '    self.__' + nodeInstanceName + ' = ModelNode({\'type\':\'' + nodeType + '\''
        for att in nodeInstance.generatedAttributes: 
          attValue = eval('self.ASGroot.listNodes[\'' + node + '\'][' + str(i) + '].' + att)
          if isinstance(attValue, ATOM3String):
             content += ', \'' + att + '\':\'' + attValue.strValue + '\''
          elif isinstance(attValue, ATOM3Integer):
             content += ', \'' + att + '\':' + str(attValue.intValue)
        content += '})\n'
        content += '    self.__rhsModel.get_nodes().append(self.__' + nodeInstanceName + ')\n'
        i = i + 1

  content += '    \n'
  content += '    # Generic nodes\n'
  
  separatorX = self.ASGroot.listNodes[Magic().separatorName][0].graphObject_.x

  for node in self.ASGroot.listNodes:
    if node.startswith(Magic().genericPrefix):
      nodeType = node
      i = 0
      for nodeInstance in self.ASGroot.listNodes[node]:
        nodeInstanceName = 'obj' + str(nodeInstance.objectNumber)
        content += '    \n'
        content += '    self.__' + nodeInstanceName + ' = ModelNode({\'type\':\'' + nodeType + '\''
        for att in nodeInstance.generatedAttributes: 
          attValue = eval('self.ASGroot.listNodes[\'' + node + '\'][' + str(i) + '].' + att)
          if isinstance(attValue, ATOM3String):
             content += ', \'' + att + '\':\'' + attValue.strValue + '\''
          elif isinstance(attValue, ATOM3Integer):
             content += ', \'' + att + '\':' + str(attValue.intValue)
        content += '})\n'
        i = i + 1
        if nodeInstance.graphObject_.x < separatorX:
          # GenericNode on LHS side
          content += '    self.__lhsModel.get_nodes().append(self.__' + nodeInstanceName + ')\n'
        else:
          # GenericNode on RHS side
          content += '    self.__rhsModel.get_nodes().append(self.__' + nodeInstanceName + ')\n'

  content += '    \n'
  content += '    # LHS node connections\n'
  content += '    \n'

  for node in self.ASGroot.listNodes:
    if node.startswith(Magic().lhsPrefix):
      for nodeInstance in self.ASGroot.listNodes[node]:
        nodeInstanceName = 'obj' + str(nodeInstance.objectNumber)
        for nodeInstanceConnection in nodeInstance.in_connections_:
          nodeInstanceConnectionName = 'obj' + str(nodeInstanceConnection.objectNumber)
          content += '    self.__' + nodeInstanceName + '.get_inConnections().append(self.__' + nodeInstanceConnectionName + ')\n'
        for nodeInstanceConnection in nodeInstance.out_connections_:
          nodeInstanceConnectionName = 'obj' + str(nodeInstanceConnection.objectNumber)
          content += '    self.__' + nodeInstanceName + '.get_outConnections().append(self.__' + nodeInstanceConnectionName + ')\n'
        content += '    \n'

  content += '    # RHS node connections\n'
  content += '    \n'

  for node in self.ASGroot.listNodes:
    if node.startswith(Magic().rhsPrefix): 
      for nodeInstance in self.ASGroot.listNodes[node]:
        nodeInstanceName = 'obj' + str(nodeInstance.objectNumber)
        for nodeInstanceConnection in nodeInstance.in_connections_:
          nodeInstanceConnectionName = 'obj' + str(nodeInstanceConnection.objectNumber)
          content += '    self.__' + nodeInstanceName + '.get_inConnections().append(self.__' + nodeInstanceConnectionName + ')\n'
        for nodeInstanceConnection in nodeInstance.out_connections_:
          nodeInstanceConnectionName = 'obj' + str(nodeInstanceConnection.objectNumber)
          content += '    self.__' + nodeInstanceName + '.get_outConnections().append(self.__' + nodeInstanceConnectionName + ')\n'
        content += '    \n'

  content += '    # Generic node connections\n'
  content += '    \n'

  for node in self.ASGroot.listNodes:
    if node.startswith(Magic().genericPrefix): 
      for nodeInstance in self.ASGroot.listNodes[node]:
        nodeInstanceName = 'obj' + str(nodeInstance.objectNumber)
        for nodeInstanceConnection in nodeInstance.in_connections_:
          nodeInstanceConnectionName = 'obj' + str(nodeInstanceConnection.objectNumber)
          content += '    self.__' + nodeInstanceName + '.get_inConnections().append(self.__' + nodeInstanceConnectionName + ')\n'

        for nodeInstanceConnection in nodeInstance.out_connections_:
          nodeInstanceConnectionName = 'obj' + str(nodeInstanceConnection.objectNumber)
          content += '    self.__' + nodeInstanceName + '.get_outConnections().append(self.__' + nodeInstanceConnectionName + ')\n'
        content += '    \n'


  content += '  def match(self, model):\n'
  content += '    nac = False\n'
  content += '    for lhsRuleNode in self.__lhsModel.get_nodes():\n'
  content += '      for node in model.get_model().get_nodes():\n'
  content += '        nacMatched = False\n'
  content += '        for rhsRuleNode in self.__rhsModel.get_nodes():\n'
  content += '          if self.__matchNode(rhsRuleNode, node):\n'
  content += '            nacMatched = True\n'
  content += '            nac = True\n'
  content += '          for n in model.get_model().get_nodes():\n'
  content += '            n.set_matched(False)\n'
  content += '            if n.get_parameters().has_key(\'label\'):\n'
  content += '              del n.get_parameters()[\'label\'] \n'
  content += '          for n in self.__rhsModel.get_nodes():\n'
  content += '            n.set_matched(False)\n'
  content += '          if nacMatched:\n'
  content += '            break\n'
  content += '        if nacMatched:\n'
  content += '          continue\n'
  content += '        if self.__matchNode(lhsRuleNode, node):\n'
  content += '          for n in self.__lhsModel.get_nodes():\n'
  content += '            n.set_matched(False)\n'
  content += '          print \'LHS rule matched\'\n'
  content += '          self.__transform(model.get_model())\n'
  content += '          print \'Model transformed\'\n'
  content += '          return True\n'
  content += '        for n in self.__lhsModel.get_nodes(): \n'
  content += '          n.set_matched(False)\n'
  content += '    if nac:\n'
  content += '      print \'NAC matched\'\n'
  content += '    else:\n'
  content += '      print \'No match for LHS rule\'\n'
  content += '    return False\n'
  content += '  \n'
  content += '  def __matchNode(self, ruleNode, node):\n'
  content += '    """\n'
  content += '    Try to recursively match a ruleNode on a node\n'
  content += '    """\n'
  content += '    attributesMatched = True\n'
  content += '    for attribute in ruleNode.get_parameters():\n'
  content += '      # Ignore these\n'
  content += '      if attribute == \'label\' or ruleNode.get_parameters()[attribute] == \'<ANY>\' or ruleNode.get_parameters()[attribute] == \'\':\n'
  content += '        continue\n'
  content += '      nodeAttribute = attribute\n'
  content += '      if attribute.endswith(\'_constraint\'):\n'
  content += '        nodeAttribute = attribute.split(\'_constraint\')[0]\n'
  content += '      if node.get_parameters().has_key(nodeAttribute):\n'
  content += '        if str(node.get_parameters()[nodeAttribute]) != str(ruleNode.get_parameters()[attribute]):\n'
  content += '          attributesMatched = False\n'
  content += '          break\n'
  content += '    if attributesMatched:\n'
  content += '      # The rule\'s node and the models node match\n'
  content += '      node.set_matched(True) \n'
  content += '      ruleNode.set_matched(True) \n'
  content += '      \n'
  content += '      # Match the in_connections\n'
  content += '      for inConnection1 in ruleNode.get_inConnections():\n'
  content += '        if not inConnection1.get_matched():\n'
  content += '          match = False\n'
  content += '          for inConnection2 in node.get_inConnections():\n'
  content += '            if not inConnection2.get_matched():\n'
  content += '              if self.__matchNode(inConnection1, inConnection2):\n'
  content += '                match = True\n'
  content += '                break\n'
  content += '          if match:\n'
  content += '            continue\n'
  content += '          node.set_matched(False) \n'
  content += '          ruleNode.set_matched(False)\n'
  content += '          return False\n'
  content += '      \n'
  content += '      # Match the out_connections\n'
  content += '      for outConnection1 in ruleNode.get_outConnections():\n'
  content += '        if not outConnection1.get_matched():\n'
  content += '          match = False\n'
  content += '          for outConnection2 in node.get_outConnections():\n'
  content += '            if not outConnection2.get_matched():\n'
  content += '              if self.__matchNode(outConnection1, outConnection2):\n'
  content += '                match = True\n'
  content += '                break\n'
  content += '          if match:\n'
  content += '            continue\n'
  content += '          node.set_matched(False) \n'
  content += '          ruleNode.set_matched(False)\n'
  content += '          return False\n'
  content += '      \n'
  content += '      # Set the matching label for this node\n'
  content += '      if ruleNode.get_parameters().has_key(\'label\'):\n'
  content += '        node.get_parameters()[\'label\'] = ruleNode.get_parameters()[\'label\']\n'
  content += '      \n'
  content += '      return True\n'
  content += '    else:\n'
  content += '      #print \'no match for \', ruleNode.get_parameters()\n'
  content += '      return False\n'
  content += '  \n'
  content += '  def __transform(self, model):\n'
  content += '    """\n'
  content += '    Try to transform a model\n'
  content += '    """\n'
  content += '    # First make the newModelPart by using the rhs model\n'
  content += '    newModelPart = copy.deepcopy(self.__rhsModel)\n'
  content += '    \n'
  content += '    modelNodes = copy.copy(model.get_nodes())\n'
  content += '    # Make sure all connections are ok\n'
  content += '    for newNode in newModelPart.get_nodes():\n'
  content += '      if newNode.get_parameters().has_key(\'label\'):\n'
  content += '        for node in modelNodes:\n'
  content += '          if node.get_parameters().has_key(\'label\'):\n'
  content += '            if newNode.get_parameters()[\'label\'] == node.get_parameters()[\'label\']:\n'
  content += '              # Start replacing connections\n'
  content += '              inConnections = copy.copy(node.get_inConnections())\n'
  content += '              for inConnection in inConnections:\n'
  content += '                if not inConnection.get_matched():\n'
  content += '                  newNode.get_inConnections().append(inConnection)\n'
  content += '                  inConnection.get_outConnections().remove(node)\n'
  content += '                  inConnection.get_outConnections().append(newNode)\n'
  content += '              outConnections = copy.copy(node.get_outConnections())\n'
  content += '              for outConnection in outConnections:\n'
  content += '                if not outConnection.get_matched():\n'
  content += '                  newNode.get_outConnections().append(outConnection)\n'
  content += '                  outConnection.get_inConnections().remove(node)\n'
  content += '                  outConnection.get_inConnections().append(newNode)\n'
  content += '              model.get_nodes().remove(node)\n'
  content += '    \n'
  content += '    # Insert newModelPart in the model\n'
  content += '    model.get_nodes().extend(newModelPart.get_nodes())\n'
  content += '    \n'
  content += '    modelNodes = copy.copy(model.get_nodes())\n'
  content += '    # Remove all remaining old parts from the model (labels that are in lhs but not in rhs)\n'
  content += '    for node in modelNodes:\n'
  content += '      if node.get_matched(): \n'
  content += '        model.get_nodes().remove(node)\n'
  content += '      elif node.get_parameters().has_key(\'label\'):\n'
  content += '        del node.get_parameters()[\'label\'] \n'
  content += '      inConnections = copy.copy(node.get_inConnections())\n'
  content += '      for inConnection in inConnections:\n'
  content += '        if inConnection.get_matched():\n'
  content += '          node.get_inConnections().remove(inConnection)\n'
  content += '          #inConnection.get_outConnections().remove(node)\n'
  content += '      outConnections = copy.copy(node.get_outConnections())\n'
  content += '      for outConnection in outConnections:\n'
  content += '        if outConnection.get_matched():\n'
  content += '          node.get_outConnections().remove(outConnection)\n' 
  content += '          #outConnection.get_inConnections().remove(node)\n'
  content += '  \n'
  content += '  def get_lhsModel(self):\n'
  content += '    return self.__lhsModel\n'
  content += '  \n'
  content += '  def get_rhsModel(self):\n'
  content += '    return self.__rhsModel\n'
  content += '  \n'
  
  file = open(compiledModelPathAndFile, 'w')
  
  file.write(content)
  
  file.close()
  
  tkMessageBox.showinfo("Compiling", 'Rule compiled successfully to ' + className + '.py', parent = self)

  return True


