"""
Contains the actions that Traffic metamodel buttonclicks should perform

Author: Dieter De Hen
"""

import tkMessageBox
import os

class Magic:
  """
  Magic variables:
    If the traffic formalism is re-named or modified, the following variables must 
    be changed accordingly. 
  """
  trafficPrefix = 'T_'

def compileModel(self):
  """ 
  Compiles a Traffic model
  """
  modelPathAndFile = self.statusbar.getState(self.statusbar.MODEL)[1][0]      
  modelPath, modelName = os.path.split(modelPathAndFile)
  modelName = modelName.split('.')[0]
  compiledModelPathAndFile = modelPathAndFile.split('_MDL.')[0] + ".py"

  className = modelName.split('_MDL')[0]

  content = '"""\n'
  content += className + '\n'
  content += '\n'
  content += 'Author: Dieter De Hen\n'
  content += '"""\n'
  content += '\n'
  content += 'from ContainerFunctionality import *\n'
  content += '\n'
  content += 'class ' + className + '(object):\n'
  content += '  \n'
  content += '  def __init__(self):\n'
  content += '    self.__model = ModelContainer()\n'
  content += '    \n'
  content += '    # Nodes\n'
  
  for node in self.ASGroot.listNodes:
    if node.startswith(Magic().trafficPrefix):
      nodeType = node
      for nodeInstance in self.ASGroot.listNodes[node]:
        nodeInstanceName = 'obj' + str(nodeInstance.objectNumber)
        content += '    \n'
        content += '    self.__' + nodeInstanceName + ' = ModelNode({\'type\':\'' + nodeType + '\'})\n'
        content += '    self.__model.get_nodes().append(self.__' + nodeInstanceName + ')\n'

  content += '    \n'
  content += '    # Node connections\n'
  content += '    \n'

  for node in self.ASGroot.listNodes:
    if node.startswith(Magic().trafficPrefix):
      for nodeInstance in self.ASGroot.listNodes[node]:
        nodeInstanceName = 'obj' + str(nodeInstance.objectNumber)
        for nodeInstanceConnection in nodeInstance.in_connections_:
          nodeInstanceConnectionName = 'obj' + str(nodeInstanceConnection.objectNumber)
          content += '    self.__' + nodeInstanceName + '.get_inConnections().append(self.__' + nodeInstanceConnectionName + ')\n'
        for nodeInstanceConnection in nodeInstance.out_connections_:
          nodeInstanceConnectionName = 'obj' + str(nodeInstanceConnection.objectNumber)
          content += '    self.__' + nodeInstanceName + '.get_outConnections().append(self.__' + nodeInstanceConnectionName + ')\n'
        content += '    \n'
  
  content += '  def get_model(self):\n'
  content += '    return self.__model\n'
  content += '  \n'
  content += '  #__model = property(get_model)\n'
  
  file = open(compiledModelPathAndFile, 'w')
  
  file.write(content)
  
  file.close()
  
  tkMessageBox.showinfo("Compiling", 'Model compiled successfully to ' + className + '.py', parent = self)
  
  return True


def generateContainerFunctionality( self ):
  """
  Generates the ContainerFunctionality.py file in the models directory
  """
  modelPathAndFile = self.statusbar.getState(self.statusbar.MODEL)[1][0]      
  modelPath, modelName = os.path.split(modelPathAndFile)
  modelContainerPathAndFile = os.path.join(modelPath, "ContainerFunctionality.py")

  content = '"""\n'
  content += 'Container class to store models\n'
  content += '\n'
  content += 'Author: Dieter De Hen\n'
  content += '"""\n'
  content += '\n'
  content += 'class ModelNode(object):\n'
  content += '\n'
  content += '  def __init__(self, parameters):\n'
  content += '    self.__inConnections = []		# list of all nodes that are connected to this node (in)\n'
  content += '    self.__outConnections = []		# list of all nodes that are connected to this node (out)\n'
  content += '    self.__parameters = parameters	# dictionairy of all parameters for this node\n'
  content += '    self.__matched = False		# flag to bookmark if this node has been matched\n'
  content += '  \n'
  content += '  def get_inConnections(self):\n'
  content += '    return self.__inConnections\n'
  content += '  \n'
  content += '  def set_inConnections(self, inConnections):\n'
  content += '    self.__inConnections = inConnections\n'
  content += '  \n'
  content += '  def add_inConnection(self, inConnection):\n'
  content += '    self.__inConnections.append(inConnection)\n'
  content += '  \n'
  content += '  def remove_inConnection(self, inConnection):\n'
  content += '    self.__inConnections.remove(inConnection)\n'
  content += '  \n'
  content += '  def get_outConnections(self):\n'
  content += '    return self.__outConnections\n'
  content += '  \n'
  content += '  def set_outConnections(self, outConnections):\n'
  content += '    self.__outConnections = outConnections\n'
  content += '  \n'
  content += '  def add_outConnection(self, outConnection):\n'
  content += '    self.__outConnections.append(outConnection)\n'
  content += '  \n'
  content += '  def remove_outConnection(self, outConnection):\n'
  content += '    self.__outConnections.remove(outConnection)\n'
  content += '  \n'
  content += '  def get_parameters(self):\n'
  content += '    return self.__parameters\n'
  content += '  \n'
  content += '  def add_parameters(self, parameters):\n'
  content += '    self.__parameters.update(parameters)\n'
  content += '  \n'
  content += '  def set_parameters(self, parameters):\n'
  content += '    self.__parameters = parameters\n'
  content += '  \n'
  content += '  def get_matched(self):\n'
  content += '    return self.__matched\n'
  content += '  \n'
  content += '  def set_matched(self, matched):\n'
  content += '    self.__matched = matched\n'
  content += '  \n'
  content += '  #__inConnections = property(get_inConnections, set_inConnections)\n'
  content += '  #__outConnections = property(get_outConnections, set_outConnections)\n'
  content += '  #__parameters = property(get_parameters)\n'
  content += '  #__matched = property(get_matched, set_matched)\n'
  content += '\n'
  content += 'class ModelContainer(object):\n'
  content += '\n'
  content += '  def __init__(self):\n'
  content += '    self.__nodes = []			# list of nodes\n'
  content += '  \n'
  content += '  def get_nodes(self):\n'
  content += '    return self.__nodes\n'
  content += '  \n'
  content += '  def set_nodes(self, nodes):\n'
  content += '    self.__nodes = nodes\n'
  content += '  \n'
  content += '  def add_nodes(self, nodes):\n'
  content += '    self.__nodes.extend(nodes)\n'
  content += '  \n'
  content += '  #__nodes = property(get_nodes, set_nodes)\n'

  file = open(modelContainerPathAndFile, 'w')
  
  file.write(content)
  
  file.close()
  
  return True

