"""
Transformation.py - Will execute the graph grammar

Author: Dieter De Hen
"""

import copy
import time

from TrafficModel import *
from GraphGrammar_GG import *

class Transformation:
  
  def __init__(self):
    self.__model = TrafficModel()

    self.__gg = GraphGrammar_GG()

  def run(self):
    """
    Perform the transformation
    """
    atleastOneRuleMatched = True
    while atleastOneRuleMatched:
      gg = self.__gg.get_GG()
      atleastOneRuleMatched = False
      for ruleNode in gg:
        exec "from " + ruleNode.get_parameters()['name'] + " import *"
        rule = eval(ruleNode.get_parameters()['name'] + '()')
        print '--- Matching rule ---'
        print rule
        if rule.match(self.__model):
          atleastOneRuleMatched = True
          break


  def save(self):
    """
    Save the new model to a file
    """

    content = '"""\n'
    content += 'Transformed_MDL.py\n'
    content += '\n'
    content += 'Author: Dieter De Hen\n'
    content += '"""\n'
    content += '\n'
    content += 'from stickylink import *\n'
    content += 'from widthXfillXdecoration import *\n'

    # Get unique imports
    uniqueImports = set([]) 
    for node in self.__model.get_model().get_nodes():
      uniqueImports.add(node.get_parameters()['type'])
    for nodeType in uniqueImports:
      content += 'from ' + nodeType + ' import *\n' 

    uniqueImports = set([]) 
    for node in self.__model.get_model().get_nodes():
      uniqueImports.add('graph_' + node.get_parameters()['type'])
    for nodeType in uniqueImports:
      content += 'from ' + nodeType + ' import *\n' 

    content += 'from ATOM3Enum import *\n' 
    content += 'from ATOM3String import *\n' 
    content += 'from ATOM3BottomType import *\n' 
    content += 'from ATOM3Constraint import *\n' 
    content += 'from ATOM3Attribute import *\n' 
    content += 'from ATOM3Float import *\n' 
    content += 'from ATOM3List import *\n' 
    content += 'from ATOM3Link import *\n' 
    content += 'from ATOM3Connection import *\n' 
    content += 'from ATOM3Boolean import *\n'
    content += 'from ATOM3Appearance import *\n'
    content += 'from ATOM3Text import *\n'
    content += 'from ATOM3Action import *\n'
    content += 'from ATOM3Integer import *\n'
    content += 'from ATOM3Port import *\n'
    content += 'from ATOM3MSEnum import *\n'
    content += '\n'
    content += 'def Transformed_MDL(self, rootNode, GenericGraphRootNode=None, TrafficRootNode=None, PN_PetriNetRootNode=None):\n'
    content += '\n'
    content += '  # --- Generating attributes code for ASG Traffic ---\n'
    content += '  if( TrafficRootNode ):\n'
    content += '    # author\n'
    content += '    TrafficRootNode.author.setValue(\'Annonymous\')\n'
    content += '    \n'
    content += '    # description\n'
    content += '    TrafficRootNode.description.setValue(\'\\n\')\n'
    content += '    TrafficRootNode.description.setHeight(15)\n'
    content += '    \n'
    content += '    # name\n'
    content += '    TrafficRootNode.name.setValue(\'\')\n'
    content += '    TrafficRootNode.name.setNone()\n'
    content += '  # --- ASG attributes over ---\n'
    content += '  \n'
    content += '  # --- Generating attributes code for ASG PN_PetriNet ---\n'
    content += '  if( PN_PetriNetRootNode ):\n'
    content += '    # author\n'
    content += '    PN_PetriNetRootNode.author.setValue(\'Annonymous\')\n'
    content += '    \n'
    content += '    # description\n'
    content += '    PN_PetriNetRootNode.description.setValue(\'\\n\')\n'
    content += '    PN_PetriNetRootNode.description.setHeight(15)\n'
    content += '    \n'
    content += '    # name\n'
    content += '    PN_PetriNetRootNode.name.setValue(\'\')\n'
    content += '    PN_PetriNetRootNode.name.setNone()\n'
    content += '  # --- ASG attributes over ---\n'
    content += '  \n'
    content += '  \n'

    for node in self.__model.get_model().get_nodes():
      obj = 'self.obj' + str(self.__model.get_model().get_nodes().index(node) + 500)
      content += '  # --- Generating code for ' + obj + ' ---\n'
      content += '  ' + obj + '=' + node.get_parameters()['type'] + '(self)\n'
      content += '  ' + obj + '.isGraphObjectVisual = True\n'
      content += '  \n'
      content += '  if(hasattr(' + obj + ', \'_setHierarchicalLink\')):\n'
      content += '    ' + obj + '._setHierarchicalLink(False)\n'
      content += '  \n'
      for par in node.get_parameters():
        if par != 'label' and par != 'type':
          content += '  # ' + par + '\n'
          content += '  ' + obj + '.' + par + '.setValue('
          if isinstance(node.get_parameters()[par], str):
            content += '\'' + node.get_parameters()[par] + '\''
          elif isinstance(node.get_parameters()[par], int):
            content += str(node.get_parameters()[par])
          content += ')\n'
          content += '  \n'
      content += '  ' + obj + '.graphClass_= graph_GenericGraphEdge\n'
      content += '  if self.genGraphics:\n'
      content += '    new_obj = graph_' + node.get_parameters()['type'] +  '(1000.0,204.5,' + obj + ')\n'
      content += '    new_obj.DrawObject(self.UMLmodel)\n'
      content += '    self.UMLmodel.addtag_withtag("GenericGraphEdge", new_obj.tag)\n'
      content += '    new_obj.layConstraints = dict() # Graphical Layout Constraints\n'
      content += '  else: new_obj = None\n'
      content += '  ' + obj + '.graphObject_ = new_obj\n'
      content += '  \n'
      content += '  # Add node to the root: rootNode\n'
      content += '  rootNode.addNode(' + obj + ')\n'
      content += '  self.globalAndLocalPostcondition(' + obj + ', rootNode)\n'
      content += '  ' + obj + '.postAction( rootNode.CREATE )\n'
      content += '  \n'

    for node in self.__model.get_model().get_nodes():
      obj1 = 'self.obj' + str(self.__model.get_model().get_nodes().index(node) + 500)
      for outConnection in node.get_outConnections():
        obj2 = 'self.obj' + str(self.__model.get_model().get_nodes().index(outConnection) + 500)
        content += '  self.drawConnections(\n'
        content += '(' + obj1 + ',' + obj2 + ',[266.0, 288.0, 322.0, 288.0],"true", 2) )\n'
        content += '  \n'

    content += 'newfunction = Transformed_MDL\n'
    content += '\n'
    content += 'loadedMMName = [\'GenericGraph\', \'Traffic_META\', \'PN_PetriNet_META\']\n'
    content += '\n'
    content += 'atom3version = \'0.3\'\n'

    file = open('Transformed_MDL.py', 'w')

    file.write(content)
  
    file.close()
    

def main():
  """
  The main function...
  """
  t = Transformation()
  startTime = time.time()
  t.run()
  print '--- Transformation done in', time.time()-startTime, 'seconds ---'
  t.save()

if __name__ == "__main__":
  main()
